<?php

namespace Intergo\SubscriptionClient;

use Intergo\BaseClient\Enums\HttpMethodEnum;
use Intergo\BaseClient\Responses\BaseResponseInterface;
use Intergo\BaseClient\Responses\ErrorResponse;
use Psr\Http\Message\ResponseInterface;

abstract class AbstractSubscriptionService
{
    protected SubscriptionClient $client;

    /** @var string */
    protected $token;

    /**
     * AbstractSubscriptionService constructor.
     * https://intergo.atlassian.net/wiki/spaces/ES/pages/2149187587/Communication+between+microservices
     *
     * @param $baseUrl
     * @param $traceId
     */
    public function __construct($baseUrl,$traceId, $token)
    {
        $this->client = new SubscriptionClient($baseUrl, $traceId);
        $this->token = $token;
    }

    /**
     * @param string $endpoint
     * @param $bodyParams
     * @param array $urlParams
     * @return BaseResponseInterface
     */
    protected function execute($endpoint, $bodyParams, array $urlParams = [])
    {
        $result = $this->client->executeAuth($endpoint, $this->token, $urlParams, $bodyParams);
        return $this->response($endpoint, $result);
    }

   /**
     * @param string $action
     * @param BaseResponseInterface $result
     * @return BaseResponseInterface
     */
    abstract public function response(string $action, BaseResponseInterface $result);

    /**
     * @return BaseResponseInterface
     */
    public function getAllPlans()
    {
        return $this->execute(__FUNCTION__, []);
    }

    /**
     * @param $ip
     * @return BaseResponseInterface
     */
    public function getPlanById($id)
    {
        return $this->execute(__FUNCTION__, [],compact('id'));
    }

    /**
     * @param $attributes
     * @return BaseResponseInterface
     */
    public function createPlan($attributes)
    {
        extract($attributes);
        unset($attributes);
        return $this->execute(__FUNCTION__, get_defined_vars());
    }

    /**
     * @param $id
     * @param $attributes
     * @return BaseResponseInterface
     */
    public function updatePlan($id, $attributes)
    {
        extract($attributes);
        unset($attributes);
        return $this->execute(__FUNCTION__, get_defined_vars(),compact('id'));
    }

    /**
     * @param $ip
     * @return BaseResponseInterface
     */
    public function deletePlan($id)
    {
        return $this->execute(__FUNCTION__, [],compact('id'));
    }

    /**
     * @return BaseResponseInterface
     */
    public function getPlanTypes()
    {
        return $this->execute(__FUNCTION__,[]);
    }

    /**
     * @param $queryString
     * @return BaseResponseInterface
     */
    public function getSubscriptionsForManager($queryString)
    {
        if (is_string($queryString)) {
            parse_str($queryString, $queryString);
        }

        extract($queryString);
        unset($queryString);

        return $this->execute(__FUNCTION__, get_defined_vars());
    }

    /**
     * @param $user_id
     * @param $plan_id
     * @return BaseResponseInterface
     */
    public function storeSubscription($user_id, $plan_id)
    {
        return $this->execute(__FUNCTION__, get_defined_vars());
    }

    /**
     * @param $ip
     * @return BaseResponseInterface
     */
    public function getSubscriptionById($id)
    {
        return $this->execute(__FUNCTION__, [],compact('id'));
    }

    /**
     * @param $ip
     * @return BaseResponseInterface
     */
    public function getSubscriptionBillingLogsBasedOnSubscriptionId($subscription_id)
    {
        return $this->execute(__FUNCTION__, [],compact('subscription_id'));
    }
}